import psycopg2
import maxminddb
import datetime
from matplotlib import pyplot as plt
import matplotlib.dates as mdates
import os

def fetchData():
	locConn = psycopg2.connect("dbname=local user=local")
	locCur = locConn.cursor()
	locCur.execute("SELECT date(month), min_data, avg_data, max_data FROM slovakia_vis_data")
	ans = locCur.fetchall()
	locCur.close()
	locConn.close()
	return ans

def visualizeData(pingData):
	dates, minVis, avgVis, maxVis = zip(*pingData)
	# Plotting
	plt.figure(figsize=(10, 6))
	fig, ax1 = plt.subplots()
	ax1.plot(dates, minVis, marker='o', linestyle='', color='r')
	ax2 = ax1.twinx()
	ax2.plot(dates, avgVis, marker='o', linestyle='', color='g')
	ax3 = ax1.twinx()
	ax3.plot(dates, maxVis, marker='o', linestyle='', color='b')
	plt.title('Average ping in Dec 2021 based on IP address addition to hosts')
	plt.xlabel('Date')
	plt.ylabel('Values')

	# Formatting x-axis with date
	plt.gca().xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
	plt.gca().xaxis.set_major_locator(mdates.YearLocator())
	plt.gcf().autofmt_xdate()
	plt.show()

locConn = psycopg2.connect("dbname=local user=local")
locCur = locConn.cursor()
remConn = psycopg2.connect("dbname=" + os.environ["RP_DBNAME"] + " user=" + os.environ["RP_USER"] + " password=" + os.environ["RP_PASSWORD"] + " host=" + os.environ["RP_HOST"])
remCur = remConn.cursor()
#Create a temporary good ip table
remCur.execute("""CREATE TEMPORARY TABLE good_ip AS
(SELECT DISTINCT ip_addr FROM ping WHERE ping_ploss >= 0 AND ping_ploss < 100)
UNION
(SELECT DISTINCT ip_addr FROM topology WHERE t_status = 'C')""")
#Vsetky data od jan 2009 do dec 2021
#Zistime avg ping z najnovsieho mesiaca pre kazde useky + pomer, kde baseline (1) bude najstarsi mesiac

#WITH relevant_ip AS (SELECT h.ip_addr FROM good_ip g JOIN hosts h ON g.ip_addr = h.ip_addr WHERE EXTRACT(month FROM h.enter_date) = 1 AND EXTRACT(year FROM h.enter_date) = 2009 AND NOT(h.ip_addr <<= '0.0.0.0/8' OR h.ip_addr <<= '0.0.0.0/32' OR h.ip_addr <<= '10.0.0.0/8' OR h.ip_addr <<= '100.64.0.0/10' OR h.ip_addr <<= '127.0.0.0/8' OR h.ip_addr <<= '169.254.0.0/16' OR h.ip_addr <<= '172.16.0.0/12' OR h.ip_addr <<= '192.0.2.0/24' OR h.ip_addr <<= '192.88.99.0/24' OR h.ip_addr <<= '192.88.99.2/32' OR h.ip_addr <<= '192.168.0.0/16' OR h.ip_addr <<= '192.0.0.0/24' OR h.ip_addr <<= '198.18.0.0/15' OR h.ip_addr <<= '198.51.100.0/24' OR h.ip_addr <<= '203.0.113.0/24' OR h.ip_addr <<= '255.255.255.255/32')) SELECT AVG(p.ping_rttmin), AVG(p.ping_rttavg), AVG(p.ping_rttmax) FROM relevant_ip r JOIN ping p ON r.ip_addr = p.ip_addr AND p.ping_date >= '2009-12-01' AND p.ping_date <= '2009-12-31' AND p.ping_ploss >= 0 AND p.ping_ploss < 100
min_data = []
avg_data = []
max_data = []
min_baseline = []
avg_baseline = []
max_baseline = []
locCur.execute("DROP TABLE IF EXISTS slovakia_vis_data")
locCur.execute("CREATE TABLE slovakia_vis_data (month timestamp, min_data numeric, avg_data numeric, max_data numeric)")
with maxminddb.open_database("./dbip-city-lite-2023-10.mmdb") as reader:
	with open("output.txt", "w") as o:
		for k in range(13):
			min_data.append([])
			avg_data.append([])
			max_data.append([])
			min_baseline.append([])
			avg_baseline.append([])
			max_baseline.append([])
			
			for i in range(1, 13):
				print("Creating " + str(i) + "/" + str(2009 + k))
				remCur.execute("DROP TABLE IF EXISTS relevant_ip")
				remCur.execute("""CREATE TEMPORARY TABLE relevant_ip AS (SELECT h.ip_addr FROM good_ip g JOIN hosts h ON g.ip_addr = h.ip_addr WHERE EXTRACT(month FROM h.enter_date) = """ + str(i) + """ AND EXTRACT(year FROM h.enter_date) = """ + str(2009 + k) + """
				AND NOT(h.ip_addr <<= '0.0.0.0/8' OR h.ip_addr <<= '0.0.0.0/32' OR h.ip_addr <<= '10.0.0.0/8'
					OR h.ip_addr <<= '100.64.0.0/10' OR h.ip_addr <<= '127.0.0.0/8' OR h.ip_addr <<=
					'169.254.0.0/16' OR h.ip_addr <<= '172.16.0.0/12' OR h.ip_addr <<= '192.0.2.0/24' OR h.ip_addr <<= '192.88.99.0/24'
					OR h.ip_addr <<= '192.88.99.2/32' OR h.ip_addr <<= '192.168.0.0/16' OR h.ip_addr <<= '192.0.0.0/24'
					OR h.ip_addr <<= '198.18.0.0/15' OR h.ip_addr <<= '198.51.100.0/24' OR h.ip_addr <<= '203.0.113.0/24'
					OR h.ip_addr <<= '255.255.255.255/32'))""")
				remCur.execute("""SELECT * FROM relevant_ip""")
				IPAddresses = remCur.fetchall()
				relevantIpAddresses = []
				for IPAddressWrapper in IPAddresses:
					record = reader.get(IPAddressWrapper[0].split("/")[0])
					if record["country"] != None:
						if record["country"]["iso_code"] != None:
							if record["country"]["iso_code"] == "SK": relevantIpAddresses.append(IPAddressWrapper[0])
				remCur.execute("DROP TABLE IF EXISTS relevant_ip_country")
				remCur.execute("""CREATE TEMPORARY TABLE relevant_ip_country (ip_addr CIDR)""")
				for IPAddress in relevantIpAddresses:
					print(IPAddress)
					remCur.execute("""INSERT INTO relevant_ip_country (ip_addr) VALUES (%s)""", (IPAddress,))
				remCur.execute("SELECT COUNT(*) FROM relevant_ip_country")
				print(remCur.fetchone()[0])
				remCur.execute("""SELECT AVG(p.ping_rttmin), AVG(p.ping_rttavg), AVG(p.ping_rttmax) FROM relevant_ip_country r JOIN ping p ON r.ip_addr = p.ip_addr AND p.ping_date >= '2021-12-01' AND p.ping_date <= '2021-12-31' AND p.ping_ploss >= 0 AND p.ping_ploss < 100""")
				data = remCur.fetchone()
				min_data[k].append(data[0])
				avg_data[k].append(data[1])
				max_data[k].append(data[2])
				min_baseline[k].append(min_data[k][i - 1] / min_data[0][0])
				avg_baseline[k].append(avg_data[k][i - 1] / avg_data[0][0])
				max_baseline[k].append(max_data[k][i - 1] / max_data[0][0])
				o.write(str(i) + "/" + str(2009 + k) + ":\nMin = " + str(min_data[k][i - 1]) + " Baseline = "  + str(min_baseline[k][i - 1]) + "\n")
				o.write(str(i) + "/" + str(2009 + k) + ":\nAvg = " + str(avg_data[k][i - 1]) + " Baseline = "  + str(avg_baseline[k][i - 1]) + "\n")
				o.write(str(i) + "/" + str(2009 + k) + ":\nMax = " + str(max_data[k][i - 1]) + " Baseline = "  + str(max_baseline[k][i - 1]) + "\n")
				locCur.execute("INSERT INTO slovakia_vis_data VALUES('" + str(2009 + k) + '-' + str(i) + "-01 00:00:00', " + str(min_data[k][i-1]) + "," + str(avg_data[k][i-1]) + "," + str(max_data[k][i-1]) + ")")
				locConn.commit()
locConn.close()
remConn.close()
visData = fetchData()
visualizeData(visData)

