import psycopg2
import maxminddb
import datetime
import matplotlib.pyplot as plt
import networkx as nx
from mpl_toolkits.basemap import Basemap
import os

def plot_graph_on_world_map(vertices, edges, year, month):
    fig, ax = plt.subplots(figsize=(15, 10))
    m = Basemap(projection='merc', llcrnrlat=-80, urcrnrlat=80,
                llcrnrlon=-180, urcrnrlon=180, resolution='i', ax=ax)
    m.drawcoastlines()
    m.drawcountries()
    m.drawmapboundary()
    G = nx.Graph()
    for vertex in vertices:
        G.add_node(vertex['name'], pos=(vertex['lon'], vertex['lat']))
    for edge in edges:
        G.add_edge(edge[0], edge[1])
    pos = nx.get_node_attributes(G, 'pos')
    pos_map = {k: m(v[0], v[1]) for k, v in pos.items()}
    nx.draw_networkx_nodes(G, pos_map, node_size=50, node_color='red', alpha=0.6)
    nx.draw_networkx_edges(G, pos_map, edge_color='blue', alpha=0.5)
    plt.title('Graph on World Map')
    plt.savefig(str(year) + "_" + str(month) + ".pdf")

remConn = psycopg2.connect("dbname=" + os.environ["RP_DBNAME"] + " user=" + os.environ["RP_USER"] + " password=" + os.environ["RP_PASSWORD"] + " host=" + os.environ["RP_HOST"])
remCur = remConn.cursor()

def generateGraph(year, month):
	thisMonth = datetime.datetime(year, month, 1)
	nextMonth = datetime.datetime(year, month + 1, 1)
	#Teraz nazbierat vsetky hrany
	remCur.execute("""SELECT t_route FROM topology WHERE t_status = 'C' AND t_date >= '""" + thisMonth.strftime('%Y-%m-%d') + """' AND t_date < '""" + nextMonth.strftime('%Y-%m-%d') + """'
	""")
	existing_edges = set()
	existing_IPs = set()
	noDataIPs = set()
	vertices = []
	# Add edges to the graph based on the routes
	with maxminddb.open_database("./dbip-city-lite-2023-10.mmdb") as reader:
		for record in remCur:
			route = record[0]  # Assuming this is a list or tuple of node identifiers
			for i in range(len(route) - 1):
				src, dest = route[i], route[i + 1]
				if not(src == '0.0.0.0/32' or dest == '0.0.0.0/32' or src == dest):
					if src not in existing_IPs and src not in noDataIPs:
						existing_IPs.add(src)
						locData = reader.get(src.split("/")[0])
						try:
							vertices.append({'name': src.split("/")[0], 'lat': locData['location']['latitude'], 'lon': locData['location']['longitude']})
						except:
							noDataIPs.add(src)
					if dest not in existing_IPs and dest not in noDataIPs:
						existing_IPs.add(dest)
						locData = reader.get(dest.split("/")[0])
						try:
							vertices.append({'name': dest.split("/")[0], 'lat': locData['location']['latitude'], 'lon': locData['location']['longitude']})
						except:
							noDataIPs.add(dest)
					
					# Check if the edge already exists before adding
					if not src in noDataIPs and not dest in noDataIPs and (src, dest) not in existing_edges:
						existing_edges.add((src.split("/")[0], dest.split("/")[0]))
	plot_graph_on_world_map(vertices, existing_edges, year, month)
	
for year in range(2009, 2022):
	for month in range(1, 13):
		generateGraph(year, month)
# Close the database cursor and connection
remCur.close()
remConn.close()		
